"use strict";
this.name = "Cabal_Common_Comms";
this.author = "Svengali";
this.copyright = "(C)2010-2013, License:CC-by";
this.description = "Secure comms channels - controlling script.";
this.version = "1.7.2";

this.startUp = function()
{
    delete this.startUp;
    this.helper = new worldScripts.Cabal_Common_Functions.Cabal_Common();
    this.commChannels = [{display:"Discard"},{display:"Open channel"}];
    this.commChannelChanged = true;
    this.patchedIDs = [];
};
/* function: addToComm()
Parameters:
display - String. Used for onscreen display.
who - Entity/String. The entity or worldScript.
ent - Boolean. Must be true for entities, blank for worldScripts.
pID - Number/String. Must be entityPersonality for entities and this.name for worldScripts.
callback - String. The function that handles the actions.
noDist - Boolean. Optional. If true keep entity entry if out of scanner range.
react - Array. Holds the names of the actions.
*/
this.addToComm = function(obj)
{
    if(obj){
        if(!obj.who || !obj.display || !obj.callback || !obj.react || !obj.hasOwnProperty("pID")){
            log(this.name,this.name+" **ERROR: Script has passed invalid settings!");
            var prop;
            for(prop in obj){log(this.name,"Settings: "+prop+" : "+obj[prop]);}
            return(false);
        }
        this.commChannels.push(obj);
        if(obj.pID) this.patchedIDs.push(obj.pID);
        this.commChannelChanged = true;
        this.checkEQ('EQ_CABAL_COMMON_COMM');
        return(true);
    }
    return(false);
};
this.commsMessageReceived = function(message,sender)
{
    if(sender && sender.scriptInfo && sender.scriptInfo.ccl_secureChannel){
        if(this.patchedIDs.indexOf(sender.entityPersonality)===-1){
            var channel = sender.scriptInfo.ccl_secureChannel;
            if(sender.script[channel]) this.checkEQ('EQ_CABAL_COMMON_COMM');
        }
    }
};
/* function: removeFromComm()
Removes object from the list.

> worldScripts.Cabal_Common_Comms.removeFromComm(this.myComm);
*/
this.removeFromComm = function(obj)
{
    if(obj){
        this.patchedIDs = this.helper.arrRemoveByValue(this.patchedIDs,obj.pID);
        this.commChannels = this.helper.arrRemoveByValue(this.commChannels,obj);
        this.commChannelChanged = true;
        return(true);
    }
    return(false);
};
/* function: changeChoicesComm()
Changes the stored actions for a specific entry.
*/
this.changeChoicesComm = function(pID,newSet)
{
    if(this.patchedIDs.indexOf(pID)===-1) return(false);
    var l = this.commChannels.length;
    if(l>2){
        while(l--){
            if(this.commChannels[l].pID===pID){
                this.commChannels[l].react=newSet;
                this.commChannelChanged = true;
                this.checkEQ('EQ_CABAL_COMMON_COMM');
                return(true);
            }
        }
    }
    return(false);
};
this.shipWillEnterWitchspace = function()
{
    this.cleanTimer(); // Avoid eating time
    this.commChannels = [{display:"Discard"},{display:"Open channel"}];
    this.commChannelChanged = false;
    if(player.ship.equipmentStatus('EQ_CABAL_COMMON_COMM')!=='EQUIPMENT_UNAVAILABLE') player.ship.removeEquipment('EQ_CABAL_COMMON_COMM');
};
this.doCleanUpTimer = function()
{
    if(!player.ship.isValid) return;
    var l = this.commChannels.length;
    if(l>2){
        while(l--){
            if(this.commChannels[l].noDist) continue;
            if(this.commChannels[l].ent){
                if(this.commChannels[l].who.isValid && this.commChannels[l].who.position.distanceTo(player.ship.position)<25600) continue;
                else {
                    if(this.commChannels[l].pID) this.patchedIDs = this.helper.arrRemoveByValue(this.patchedIDs,this.commChannels[l].pID);
                    this.commChannels = this.helper.arrRemoveByValue(this.commChannels,this.commChannels[l]);
                    this.commChannelChanged = true;
                    break; // Avoid eating time
                }
            } else if(this.commChannels[l].eID){
                if(this.commChannels[l].eID.isValid && this.commChannels[l].eID.position.distanceTo(player.ship.position)<25600) continue;
                else {
                    if(this.commChannels[l].pID) this.patchedIDs = this.helper.arrRemoveByValue(this.patchedIDs,this.commChannels[l].pID);
                    this.commChannels = this.helper.arrRemoveByValue(this.commChannels,this.commChannels[l]);
                    this.commChannelChanged = true;
                    break; // Avoid eating time
                }
            }
        }
    } else this.cleanTimer(); // Avoid eating time
    return;
};
this.equipmentDamaged = this.equipmentDestroyed = function(equipmentKey)
{
    if(!player.ship.isValid) return;
    if(equipmentKey==='EQ_CABAL_COMMON_COMM') this.checkEQ('EQ_CABAL_COMMON_COMM');
};
this.checkEQ = function()
{
    if(player.ship.equipmentStatus('EQ_CABAL_COMMON_COMM')==='EQUIPMENT_UNAVAILABLE') player.ship.awardEquipment('EQ_CABAL_COMMON_COMM');
    else if(player.ship.equipmentStatus('EQ_CABAL_COMMON_COMM')==='EQUIPMENT_DAMAGED') player.ship.setEquipmentStatus('EQ_CABAL_COMMON_COMM','EQUIPMENT_OK');
    if(!this.cleanUpTimer) this.cleanUpTimer = new Timer(this,this.doCleanUpTimer,0,25);
    else this.cleanUpTimer.start();
    return;
};
this.shipDied = function()
{
    this.cleanTimer();
};
this.cleanTimer = function()
{
    if(this.cleanUpTimer){
        this.cleanUpTimer.stop();
        delete this.cleanUpTimer;
    }
    return;
};
