"use strict";
this.name = "Cabal_Common_OXPStrength";
this.author = "Svengali";
this.copyright = "(C)2010-2013, License:CC-by";
this.description = "Checks and/or cleans out entities and suspends OXPs.";
this.version = "1.7.1";

this.ccl_strengthExclEntByName = []; // Array of entity names to be excluded
this.startUp = function()
{
	delete this.startUp; // PhantorGorth' chain handling
	this.checkOXPStrength = new this.Cabal_Common_OXPStrengthAPI(); // Get instance
	this.helper = new worldScripts.Cabal_Common_Functions.Cabal_Common();
	this.gJump = false; // Flag for galactic jump
	this.current = null; // Holds current settings object or null
	this.watchLocations = [[],[],[],[],[],[],[],[]]; // Storage for settings splitted in galaxies to keep loops small
	this.resetValues(); // Default settings
	this.leftSystem = system.ID; // For interstellar checks
	this.suspendedOXPs = []; // Holds suspended OXPs
	for(var w in worldScripts){
		if(!w) continue;
		var s = worldScripts[w];
		if(!s) continue;
		for(var prop in s){
			var p = s[prop];
			if(!prop || !p) continue;
			if(typeof(p)==="function"){
				var temp = p.toString();
				if(temp.search(/restoreSubEntities/gi)!==-1) this.checkOXPStrength.wayoff.push(w.toLowerCase());
				if(temp.search(/EQ_RENOVATION/gi)!==-1) this.checkOXPStrength.wayoff.push(w.toLowerCase());
			}
		}
	}
};
this.guiScreenChanged = this.alertConditionChanged = this.missionScreenOpportunity = function()
{
	delete this.guiScreenChanged;
	delete this.alertConditionChanged;
	delete this.missionScreenOpportunity;
	// Excluded entities?
	if(this.ccl_strengthExclEntByName.length){
		for(var i=0;i<this.ccl_strengthExclEntByName.length;i++){
			if(typeof(this.ccl_strengthExclEntByName[i])==="string") this.checkOXPStrength.binAdd(this.ccl_strengthExclEntByName[i],true);
			else {
				for(var j=0;j<this.ccl_strengthExclEntByName[i].length;j++){
					this.checkOXPStrength.binAdd(this.ccl_strengthExclEntByName[i][j],true);
				}
			}
		}
	}
	this.checkWatchLocations(system.ID,false);
};
// Interface for other OXPS to register settings
this.registerLocation = function(obj)
{
	if(obj && obj.hasOwnProperty("Gal") && obj.hasOwnProperty("ID") && obj.hasOwnProperty("ISS") && obj.CSS && obj.CSP){
		this.watchLocations[obj.Gal].push(obj);
		return(true);
	}
	return(false);
};
// Oktis approach - callback
this.suspendOXP = function()
{
	if(this.current && this.suspendedOXPs.length) return(true);
	return(false);
};
this.playerWillSaveGame = function()
{
	this.tempS = missionVariables.CCL_OXPStrength;
	missionVariables.CCL_OXPStrength = 9; //Save only default
	this.guiScreenChanged = function(){
		missionVariables.CCL_OXPStrength = this.tempS;
		delete this.guiScreenChanged;
	}
};
// Defaults
this.resetValues = function(full)
{
	this.checkOXPStrength.checkCoef = 999999; // Minimum coef
	this.checkOXPStrength.checkKill = 999999; // Minimum coef for removing
	this.checkOXPStrength.logIt = false; // Logging flag
	this.checkOXPStrength.checkLocation = 0; // Location to be checked
	missionVariables.CCL_OXPStrength = 9; // For conditions in shipdata
	this.current = null;
	if(full) this.suspendedOXPs = [];
	return;
};
this.playerStartedJumpCountdown = function(jump)
{
	if(jump==="galactic") this.gJump = true;
};
this.playerCancelledJumpCountdown = this.playerJumpFailed = function()
{
	this.gJump = false;
};
// Interstellar and normal space checks
this.shipWillExitWitchspace = this.shipWillLaunchFromStation = function()
{
	this.gJump = false;
	if(system.isInterstellarSpace) this.checkWatchLocations(this.leftSystem,true);
	else this.checkWatchLocations(system.ID,false);
	if(this.current) this.startCleaning(this.current);
	else this.resetValues(1);
};
// Reset and prepare next checks. MV has to be set before jumping to prevent spawning by populator
this.shipWillEnterWitchspace = function()
{
	if(this.repeatTimer){
		this.repeatTimer.stop();
		delete this.repeatTimer;
	}
	this.leftSystem = system.ID;
	if(this.current && !this.current.Permanent) this.watchLocations[galaxyNumber] = this.checkOXPStrength.arrRemoveByValue(this.watchLocations[galaxyNumber],this.current);
	this.resetValues();
	if(!this.gJump){
		if(player.ship.scriptedMisjump) this.checkWatchLocations(this.leftSystem,true);
		else this.checkWatchLocations(player.ship.targetSystem,false);
		this.current = null;
	} else {
		var loc = player.ship.galacticHyperspaceFixedCoordsInLY.toArray(); // Rounding!!!
		for(var r=0;r<3;r++){
			loc[r] = parseFloat(loc[r].toFixed(1));
		}
		this.checkWatchLocations(loc,false);
	}
};
// Check if it should be activated
this.checkWatchLocations = function(ID,ISS)
{
	var c=[],temp;
	// Grab subset
	if(this.gJump){
		if(galaxyNumber<7) c = this.watchLocations[galaxyNumber+1];
		else c = this.watchLocations[0];
	} else c = this.watchLocations[galaxyNumber];
	for(var i=0;i<c.length;i++){
		if(c[i].ID===ID || (this.gJump && c[i].GJump && c[i].GJump[0]===ID[0] && c[i].GJump[1]===ID[1])){
			temp = null;
			if(c[i].CSS && c[i].CSP && !worldScripts[c[i].CSS][c[i].CSP]) continue; // Check status
			if((system.isInterstellarSpace || player.ship.scriptedMisjump) && ISS && c[i].ISS) temp = c[i];
			else if(!system.isInterstellarSpace && !player.ship.scriptedMisjump && !ISS && !c[i].ISS) temp = c[i];
			else if(system.isInterstellarSpace && !player.ship.scriptedMisjump && !ISS && !c[i].ISS) temp = c[i];
			if(temp){
				if(!this.current) this.current = temp;
				else {
					if(temp.Strength && temp.Strength<this.current.Strength || !this.current.hasOwnProperty("Strength")) this.current.Strength = temp.Strength;
					if(temp.checkC && temp.checkC<this.current.checkC || !this.current.hasOwnProperty("checkC")) this.current.checkC = temp.checkC;
					if(temp.checkK && temp.checkK<this.current.checkK || !this.current.hasOwnProperty("checkK")) this.current.checkK = temp.checkK;
					if(temp.delay && temp.delay<this.current.delay || !this.current.hasOwnProperty("delay")) this.current.delay = temp.delay;
					if(temp.repeat && temp.repeat>this.current.repeat || !this.current.hasOwnProperty("repeat")) this.current.repeat = temp.repeat;
					if(temp.locate && temp.locate!==this.current.locate || !this.current.hasOwnProperty("locate")) this.current.locate = 0;
					if(temp.logging && !this.current.logging) this.current.logging = 1;
					if(temp.Suspend) this.current.Suspend = this.suspendedOXPs.concat(temp.Suspend);
				}
			}
		}
	}
	if(this.current){
		if(this.current.Strength) missionVariables.CCL_OXPStrength = this.current.Strength; // Set mV for strength
		if(this.current.Suspend && this.current.Suspend.length){
			for(var s=0;s<this.current.Suspend.length;s++){
				c = this.current.Suspend[s];
				if(this.suspendedOXPs.indexOf(c)!==-1) continue;
				if(worldScripts[c] && worldScripts[c].addToCallbacks){
					worldScripts[c].addToCallbacks({worldScript:this.name,callBack:"suspendOXP"});
					this.suspendedOXPs.push(this.current.Suspend[s]);
				}
			}
		}
	} else {
		if(this.infoTimer){this.infoTimer.stop(); delete this.infoTimer;}
		if(this.repeatTimer){this.repeatTimer.stop(); delete this.repeatTimer;}
		if(this.suspendedOXPs.length){
			for(var s=0;s<this.suspendedOXPs.length;s++){
				c = this.suspendedOXPs[s];
				if(worldScripts[c] && worldScripts[c].removeFromCallbacks){
					worldScripts[c].removeFromCallbacks({worldScript:this.name,callBack:"suspendOXP"});
				}
			}
			this.suspendedOXPs = [];
		}
	}
	return;
};
// Set cleaning params
this.startCleaning = function(obj)
{
	if(!obj.checkC) return;
	else {
		this.checkOXPStrength.checkCoef = obj.checkC;
		if(obj.checkK) this.checkOXPStrength.checkKill = obj.checkK;
	}
	if(obj.logging) this.checkOXPStrength.logIt = obj.logging;
	// One shot timer
	if(!this.infoTimer){
		if(obj.delay) this.infoTimer = new Timer(this,this.checkAll,obj.delay);
		else this.infoTimer = new Timer(this,this.checkAll,10);
	}
	if(obj.locate) this.checkOXPStrength.checkLocation = obj.locate;
	// Repeat timer
	if(obj.repeat){
		if(!this.repeatTimer){
			if(obj.delay) this.repeatTimer = new Timer(this,this.checkAll,0,10+obj.delay);
			else this.repeatTimer = new Timer(this,this.checkAll,0,10);
		}
	}
	return;
};
this.checkAll = function()
{
	if(guiScreen==="GUI_SCREEN_LOAD"){
		if(this.infoTimer){this.infoTimer.stop(); delete this.infoTimer;}
		if(this.repeatTimer){this.repeatTimer.stop(); delete this.repeatTimer;}
		return;
	}
	this.checkOXPStrength.checkPower();
	if(this.infoTimer){this.infoTimer.stop(); delete this.infoTimer;}
	if(this.current.repeat) this.current.repeat--;
	else if(this.repeatTimer){this.repeatTimer.stop(); delete this.repeatTimer;}
	return;
};
// API
this.Cabal_Common_OXPStrengthAPI = function(){this._cclstrength = null;}
Cabal_Common_OXPStrengthAPI.prototype = {
	// Threshold distance to MainStation, WP or the lanes WP,WS,PS
	maxDist: 50000,
	// Threshold energy
	checkE: 800,
	// Threshold speed
	checkS: 500,
	// Threshold standard missiles
	checkM: 6,
	// Threshold escorts
	checkEsc: 4,
	// Threshold for putting in list
	checkCoef: 999999,
	// Threshold for killing
	checkKill: 999999,
	// Act only for location
	checkLocation: 0,
	logIt: false,
	weapons: ["EQ_WEAPON_PULSE_LASER",1,"EQ_WEAPON_BEAM_LASER",3,"EQ_WEAPON_MILITARY_LASER",5,"EQ_WEAPON_MINING_LASER",4,"EQ_WEAPON_THARGOID_LASER",8,"EQ_WEAPON_TWIN_PLASMA_CANNON",4],
	smissile: ["EQ_MISSILE",0,"EQ_HARDENED_MISSILE",0.6,"EQ_ARMOURY_STANDARD_MISSILE",0.1,"EQ_ARMOURY_HARD_MISSILE",0.7,"EQ_MILITARY_MISSILE",0.8,"EQ_RMB_CASCADE_MISSILE",0.6,
		"EQ_RMB_FRAG_MISSILE",0.4,"EQ_RMB_LAW_MISSILE",0.5,"EQ_RMB_THARGOID_MISSILE",0.1,"EQ_RMB_INTERCEPT_MISSILE",0.3,"EQ_RMB_OVERRIDE_MISSILE",0.3,"EQ_HARPOON_NUKE1_MISSILE",0.7,
		"EQ_HARPOON_ABM_MISSILE",0.5,"EQ_MANCHI_MISSILE",0.4,"EQ_VECTOR_FUNA_MISSILE",1.1,"EQ_VECTOR_FUNB_MISSILE",1.1,"EQ_VECTOR_FUNC_MISSILE",1.1,"EQ_KILLIT_MISSILE",2.0,
		"EQ_CT_MINE",1.2,"EQ_ECD_MINE",1.1],
	srole: ["sunskim-trader",0.6,"trader",0.6,"hunter",1.1,"police",1.1,"wingman",1.1,"interceptor",1.1,"pirate",1.1,"blackdog",1.1,"scavenger",0.3,"shuttle",0.2,"miner",0.2,
		"hermit-ship",0.4,"escort",0.9,"thargoid",0.9,"tharglet",0.9],
	storage: [],
	wayoff : [],
	checkSpaceLane: function(entity){
		if(system.isInterstellarSpace){
			if(this.checkLocation){
				var d = this.maxDist+1;
				return([0,d,entity.position.distanceTo(new Vector3D([0,0,0])),d,d,d]);
			} else return([-1,0,0,0,0,0]);
		}
		if(!system.sun || system.sun.isGoingNova || system.sun.hasGoneNova) return([1,0,0,0,0,0]);
		var distMS = entity.position.distanceTo(system.mainStation.position);
		var distWP = entity.position.distanceTo(new Vector3D([0,0,0]));
		var distLWP = Math.sqrt(entity.position.x*entity.position.x+entity.position.y*entity.position.y);
		var lanePS = new Vector3D(system.mainPlanet.position.subtract(system.sun.position));
		var SEV = new Vector3D(entity.position.subtract(system.sun.position));
		var distS = SEV.magnitude();
		var distLPS = distS*Math.sin(lanePS.angleTo(SEV));
		var laneWS = new Vector3D(new Vector3D([0,0,0]).subtract(system.sun.position));
		var distLWS = distS*Math.sin(laneWS.angleTo(SEV));
		if(distMS>this.maxDist && distWP>this.maxDist && distLWP>this.maxDist && distLPS>this.maxDist && distLWS>this.maxDist) return([2,Math.floor(distMS),Math.floor(distWP),Math.floor(distLWP),Math.floor(distLPS),Math.floor(distLWS)]);
		return([0,Math.floor(distMS),Math.floor(distWP),Math.floor(distLWP),Math.floor(distLPS),Math.floor(distLWS)]);
	},
	checkSubs: function(entity){
		var tur=0,wea=0,slots=0,w=0,wtype="",s=0,subwtype="",temp;
		if(entity.forwardWeapon){slots++; temp = this.weapons[this.weapons.indexOf(entity.forwardWeapon.equipmentKey)+1]; wtype += ""+temp; w += temp;}
		if(entity.aftWeapon){slots++; temp = this.weapons[this.weapons.indexOf(entity.aftWeapon.equipmentKey)+1]; wtype += ""+temp; w += temp;}
		if(entity.portWeapon){slots++; temp = this.weapons[this.weapons.indexOf(entity.portWeapon.equipmentKey)+1]; wtype += ""+temp; w += temp;}
		if(entity.starboardWeapon){slots++; temp = this.weapons[this.weapons.indexOf(entity.starboardWeapon.equipmentKey)+1]; wtype += ""+temp; w += temp;}
		if(entity.isPlayer && !slots){slots=4; w=32; wtype="0000";} // All null, treat as highest
		if(!entity.subEntityCapacity) return([slots,0,0,w,wtype,0,0]);
		for(var prop in entity.subEntities){
			if(entity.subEntities[prop].status==="STATUS_ACTIVE") tur += 4;
			if(entity.subEntities[prop].forwardWeapon){wea++; temp = this.weapons[this.weapons.indexOf(entity.subEntities[prop].forwardWeapon.equipmentKey)+1]; subwtype += ""+temp; s += temp;}
			if(entity.subEntities[prop].aftWeapon){wea++; temp = this.weapons[this.weapons.indexOf(entity.subEntities[prop].aftWeapon.equipmentKey)+1]; subwtype += ""+temp; s += temp;}
			if(entity.subEntities[prop].portWeapon){wea++; temp = this.weapons[this.weapons.indexOf(entity.subEntities[prop].portWeapon.equipmentKey)+1]; subwtype += ""+temp; s += temp;}
			if(entity.subEntities[prop].starboardWeapon){wea++; temp = this.weapons[this.weapons.indexOf(entity.subEntities[prop].starboardWeapon.equipmentKey)+1]; subwtype += ""+temp; s += temp;}
		}
		if(!tur && entity.subEntityCapacity>2 && entity.subEntities && entity.subEntities.length<3) tur = entity.subEntityCapacity*4; // no cheats
		return([slots,tur,wea,w,wtype,s,subwtype]);
	},
	checkMissiles: function(entity){
		if(!entity.missileCapacity) return(0);
		var slot,mtype=0;
		for(var prop in entity.missiles){
			if(entity.missiles[prop].equipmentKey){
				slot = this.smissile.indexOf(entity.missiles[prop].equipmentKey+1);
				if(slot===-1) mtype += 2;
				else mtype += slot;
			}
		}
		if(!mtype){
			if(entity.missileCapacity>this.checkM) return([1,entity.missileCapacity]);
			else return(0);
		}
		return([(1+mtype)*entity.missileCapacity,entity.missileCapacity]);
	},
	checkRole: function(entity){
		var r = entity.primaryRole,mul=1;
		var ind = this.srole.indexOf(r);
		if(ind!==-1) mul = this.srole[ind+1];
		return(mul);
	},
	checkPower: function(p){
		if(!player.ship.isValid) return(0);
		function isBad(entity){return(entity.isShip && entity.isValid && (!entity.isStation || (entity.isStation && !entity.isMainStation)) && entity.maxEnergy>this.checkE || entity.maxSpeed>this.checkS || this.checkSubs(entity) || this.checkMissiles(entity) || (entity.escorts && entity.escorts.length>this.checkEsc));}
		var ents,temp,en,sp,wep,def,coef,mis,esc,dist,rol,bomb=false;
		this.storage = [];
		if(p){
			if(p.isPlayer) ents = [player.ship];
			else if(p.isValid) ents =[p];
			else return(0);
		} else ents = system.filteredEntities(this,isBad,player.ship);
		if(ents.length){
			for(var i=0;i<ents.length;i++){
				if(!ents[i].name) continue;
				if(!p || (p && !p.isPlayer)){
					var exFlag=this.binContains(ents[i].name);
					if(exFlag) continue;
				}
				en = ents[i].maxEnergy;
				if(ents[i].isStation && !ents[i].maxSpeed) coef = Math.log(en);
				else {
					if(en>256) en += Math.sqrt(en);
					if(p && p.isPlayer){
						en += ents[i].maxAftShield*ents[i].aftShieldRechargeRate;
						en += ents[i].maxForwardShield*ents[i].forwardShieldRechargeRate;
					} else if(en>800) en *= 2;
					coef = Math.sqrt(en);
				}
				sp = ents[i].maxSpeed;
				if(ents[i].equipmentStatus("EQ_FUEL_INJECTION")==="EQUIPMENT_OK") sp *= 4;
				if(ents[i].isPiloted && !ents[i].isStation && ents[i].maxSpeed && ents[i].maxThrust>40) sp *= Math.log(ents[i].maxThrust);
				if(sp) coef += Math.sqrt(sp);
				else sp = 0;
				temp = this.checkSubs(ents[i]);
				if(typeof(temp)!=="number"){
					if(this.logIt) wep = "L:"+temp[0]+" T:"+temp[1]+" S:"+temp[2]+" "+temp[4]+","+temp[6];
					if(temp[0]) coef += Math.pow(1.4,temp[0])*Math.log(temp[3]); // Standard Laser
					if(temp[1]) coef += Math.min(Math.pow(1.4,temp[1])*2,0xfffff); // Turrets
					if(temp[2]) coef += Math.min(Math.pow(1.5,temp[2])*(2*Math.log(temp[5])),0xfffff); // Subent Laser
				} else if(this.logIt) wep = "L:0 T:0 S:0 -,-";
				temp = this.checkMissiles(ents[i]);
				if(temp[0]){
					coef += Math.pow(temp[1],1.7);
					mis = temp[1];
				} else mis = 0;
				if(ents[i].escorts && ents[i].escorts.length) esc = ents[i].escorts.length;
				else esc = 0;
				coef += Math.pow(esc,1.8);
				if(ents[i].isStation){
					def = ents[i].dockedDefenders;
				} else def = 0;
				coef += Math.pow(def,1.8);
				if(p && p.isPlayer){
					coef *= 1.5;
					if(missionVariables.targetAutolock==="TRUE") coef += 5;
					var eq = ["EQ_ECM","EQ_ENERGY_BOMB","EQ_TARGET_AUTOLOCK","EQ_ENERGY_UNIT","EQ_NAVAL_ENERGY_UNIT","EQ_CLOAKING_DEVICE","EQ_QC_MINE","EQ_SHIELD_BOOSTER","EQ_NAVAL_SHIELD_BOOSTER","EQ_MILITARY_JAMMER",
						"EQ_MILITARY_SCANNER_FILTER","EQ_SHIELD_ENHANCER","EQ_AMS","EQ_EEU","EQ_IRONHIDE_FORWARD","EQ_IRONHIDE_AFT","EQ_VORTEX_TURRET"];
					var eqi = eq.length;
					while(eqi--){if(EquipmentInfo.infoForKey(eq[eqi])!==null && player.ship.equipmentStatus(eq[eqi])==="EQUIPMENT_OK") coef += 5;}
					dist = "-";
					rol = 1;
				} else {
					dist = this.checkSpaceLane(ents[i]);
					if(dist[0]===0){
						if(this.checkLocation && dist[this.checkLocation]>this.maxDist) continue;
						if(dist[1]<this.maxDist) coef *= 1.5; // Main Station
						else if(dist[2]<this.maxDist) coef *= 1.5; // WP
						else if(dist[3]<this.maxDist) coef *= 1.3; // Lane WP
						else if(dist[4]<this.maxDist) coef *= 1.1; // Lane PS
						else if(dist[5]<this.maxDist) coef *= 1.2; // Lane WS
					}
					if(dist[0]===2) coef *= 1/Math.log(dist[3]-this.maxDist);
					rol = this.checkRole(ents[i]);
					coef *= rol;
				}
				var eq = ents[i].equipment,eqpoints=1;
				for(var eqq=0;eqq<eq.length;eqq++){
					var reg = eq[eqq].equipmentKey;
					var des = eq[eqq].description;
					if(reg.search(/TARGET/gi)!==-1) eqpoints *= 1.15;
					if(reg.search(/MISSILE/gi)!==-1) eqpoints *= 1.15;
					if(reg.search(/ENERGY/gi)!==-1) eqpoints *= 1.25;
					if(reg.search(/SHIELD/gi)!==-1) eqpoints *= 1.25;
					if(reg.search(/BOOST/gi)!==-1) eqpoints *= 1.4;
					if(reg.search(/NAVAL/gi)!==-1) eqpoints *= 1.4;
					if(reg.search(/TURRET/gi)!==-1) eqpoints *= 1.5;
					if(des.search(/ARMOUR/gi)!==-1) eqpoints *= 1.5;
					if(reg.search(/LASER/gi)!==-1 || des.search(/LASER/gi)!==-1) coef *= 2.5;
					if(reg.search(/LAZER/gi)!==-1 || des.search(/LAZER/gi)!==-1) coef *= 2.5;
				}
				coef += eqpoints;
				if(ents[i].equipmentStatus("EQ_CLOAKING_DEVICE")==="EQUIPMENT_OK") coef *= 1.3;
				if(ents[i].equipmentStatus("EQ_MILITARY_JAMMER")==="EQUIPMENT_OK") coef *= 1.5;
				if(ents[i].equipmentStatus("EQ_MILITARY_SCANNER_FILTER")==="EQUIPMENT_OK") coef *= 1.5;
				coef = Math.floor(coef);
				if(coef>this.checkCoef){
					if(this.logIt) this.storage.push("COEF:"+this.pad(coef,6)+" E:"+this.pad(en,7)+" : S:"+this.pad(sp,5)+" : C:"+ents[i].scanClass+" : "+ents[i].name+" ("+ents[i].primaryRole+") : W:"+wep+" : M:"+mis+" : Esc:"+esc+" : Def:"+def+" : dist:"+dist+" : roleMul:"+rol);
					if(!p && coef>this.checkKill && ents[i].isValid){
						if(ents[i].scriptInfo && ents[i].scriptInfo.ccl_missionShip==="true") continue;
						if(ents[i].script && ents[i].script.ccl_missionShip) continue;
						if(ents[i].AI && ents[i].AI==="cabal_common_leaveHotspotAI.plist") continue;
						if(ents[i].isMainStation || (ents[i].isStation && !ents[i].distanceTravelled)) continue;
						var custSc = false;
						if(!ents[i].script || (ents[i].script && (ents[i].script.hasOwnProperty("shipRemoved") || ents[i].script.hasOwnProperty("entityDestroyed")))) custSc = true;
						if(ents[i].status!=="STATUS_IN_FLIGHT") continue; // Avoid early calls resulting in undefined properties
						var rDist = ents[i].position.distanceTo(player.ship.position);
						if(rDist>Math.pow(ents[i].collisionRadius,2) && rDist>25600){
							if(ents[i].escorts && ents[i].escorts.length){
								var es = ents[i].escorts.length;
								while(es--){
									if(!ents[i].escorts[es].isValid) continue;
									if(!ents[i].escorts[es].script || ents[i].escorts[es].script.hasOwnProperty("shipRemoved") || ents[i].escorts[es].script.hasOwnProperty("entityDestroyed")) ents[i].escorts[es].remove(true);
									else ents[i].escorts[es].remove();
								}
							}
							if(custSc) ents[i].remove(true);
							else ents[i].remove();
							if(this.logIt) log("KILL","Kill: "+ents[i].name);
						} else {
							if(rDist>3000){
								if(this.logIt) log("KILL","Bomb: "+ents[i].name);
								if(!bomb){
									ents[i].spawn("cabal_common_oxps_mine",2);
									bomb = true;
								} else ents[i].spawn("cabal_common_oxps_minesub",2);
								if(custSc) ents[i].remove(true);
								else ents[i].remove();
							} else {
								if(ents[i].maxSpeed>100){
									ents[i].setScript("oolite-default-ship-script.js");
									ents[i].switchAI("cabal_common_leaveHotspotAI.plist");
									if(this.logIt) log("KILL","Switching Script/AI for: "+ents[i].name);
								} else if(this.logIt) log("KILL","Not possible. Player close to: "+ents[i].name);
							}
						}
					}
				}
			}
		}
		if(!p){
			if(this.logIt){
				if(system.isInterstellarSpace) log("Check","System: Interstellar.");
				else log("Check","System: "+system.name+" ID:"+system.ID+" Gov:"+system.government+" Eco:"+system.economy);
				log("Check","COEF=coefficient, E=energy, S=speed, C=scanClass, name(role), W=weapons (Lasers,Turrets,SubentLaser), M=missiles, Esc=escorts, Def=Defenders, dist=distance Station,WP,Lanes(WP,WS,PS), roleMul:multiplier.");
				var list = this.storage.sort();
				for(var i=0;i<list.length;i++){log("Check",list[i]);}
			}
		} else {
			var boo = player.ship.name.toLowerCase();
			if(this.wayoff.length){
				var patt1 = new RegExp(boo);
				for(var i=0;i<this.wayoff.length;i++){
					if(patt1.test(this.wayoff[i])) coef *= 2;
				}
			}
			if(this.logIt){
				if(p.isPlayer) log("Check","Playership checked in with coef: "+coef);
				else log("Check","Entity "+p.name+" checked in with coef: "+coef);
				log("Check","COEF=coefficient, E=energy, S=speed, C=scanClass, name(role), W=weapons (Lasers,Turrets,SubentLaser), M=missiles, Esc=escorts, Def=Defenders, dist=distance Station,WP,Lanes(WP,WS,PS), roleMul:multiplier.");
				log("Check",this.storage);
			}
			return(parseFloat(coef));
		}
		return(0);
	},
	pad: function(n,len){
		n = n.toFixed(0);
		var s = n.toString();
		if(s.length<len) s = ("0000000000"+s).slice(-len);
		return s;
	},
	arrRemoveByValue: function(arr,val){
		for(var i=0;i<arr.length;i++){
			if(arr[i]===val){
				arr.splice(i,1);
				break;
			}
		}
		return(arr);
	},
	binAdd: function(key,value){
		var node = {key: key,left: null,right: null, value: value},current;
		if(this._cclstrength === null) this._cclstrength = node;
		else {
			current = this._cclstrength;
			while(true){
				if(key<current.key){
					if(current.left===null){current.left = node; break;}
					else current = current.left;
				} else if(key>current.key){
					if(current.right===null){current.right = node; break;}
					else current = current.right;
				} else break;
			}
		}
		return;
	},
	binContains: function(key){
		var found = false,current = this._cclstrength;
		while(!found && current){
			if(key<current.key) current = current.left;
			else if(key>current.key) current = current.right;
			else found = true;
		}
		if(!found) return (false);
		return (current.value);
	}
};
